	PROGRAM MAIN

c	11_19.06 - change to zenith geometry for downwelling radiance
c	07_20.06 - change to LIDORTV3

	double precision sza,cloud_ssa
        parameter (cloud_ssa=0.99999)
	character csza*2
        parameter (sza=46.0D+00,csza='46')
        double precision salbedo,cld_toa	! at salbedo and pscale(1)=1.0
        parameter (salbedo=0.05)
	character csp*4
 	parameter (csp='so2.')
c	parameter (csp='ozn.')
c	parameter (csp='no2.')

	integer ip,nprs
        parameter (nprs=2)	! number of Pressure levels in Rayleigh atmosphere to calculate Ia(nprs), Sb(nprs) and T(nprs)
				! or single pressure level for wf calculation (call wf(..)
                            	! This is done by scaling tau_ray                                                           
        double precision pscale(nprs)
        data pscale/1.0,0.6/	!  Pg=1 [atm],   Pc=0.6 [atm]
        double precision ray_sb(nprs),ray_tb(nprs),ray_a0(nprs)
        double precision rc,asym,delt,tcl,tcl_star,rc_star,r_sys,
     &                   ler(nprs)
        parameter (asym=0.859)	! asymetry parameter for C1 cloud [Deirmendjian 1969], r_max=15um

 	call rad_ray(csp,sza,'tc00',nprs,pscale,		! Rayleigh Ia,T,Sb calculation
c	call rad_ray(csp,sza,'tc01',nprs,pscale,		! Rayleigh Ia,T,Sb calculation
     &              ray_a0,ray_sb,ray_tb)
	WRITE(57,*        )  '        Rayleigh Atmosphere'
        WRITE(57,*        ) ' Psurf       I(LER=0)        Sb	     T'

        do ip=1,nprs
	WRITE(57,'(4f13.9)')
     &        pscale(ip),ray_a0(ip),ray_sb(ip),ray_tb(ip)        
        enddo
	close(57)
        call system('mv fort.57 rad_ray46')

c	Cloud WF calculation for Ps(nprs)                   
 	call wf(cloud_ssa,csp,sza,salbedo,'tc00',1,pscale,		! input parameters: salbedo=0.05, nprs=1, pscale(1)=1.0
     &                 cld_toa,tcl)		! output parameters: cld_toa, plus wf_tcl01 - weighting function

c	Cloud albedo calculation: Rc  (two-stream approximation)
	rc_star=0.0
	r_sys = 0.0
        tcl_star=(1.-asym)*tcl
        rc=tcl_star/(tcl_star + 2.0*dcos(sza*0.0174533))
	if ( rc .gt. 0.0 ) then
        rc_star=tcl_star*(1.-0.5*tcl_star*dlog(1.+2./tcl_star))
        r_sys=rc + (1.-rc)*salbedo*(1.-rc_star)/(1.-salbedo*rc_star)
	endif

c	calculating corresponding LER valies for different Rayleigh pressure levels:
 	WRITE(57,*        )  '        Rayleigh Atmosphere: tc00'
        WRITE(57,*        ) ' Ps   Rs     cld_toa    Ia      Sb       T    
     &   	Rc    Rc*    Rsys	LER'
	do ip=1,nprs
	delt=cld_toa - ray_a0(ip)
        ler(ip)=delt/(ray_tb(ip) + delt*ray_sb(ip) )
          WRITE(57,'(2f6.2,8f9.5)')pscale(ip),salbedo,cld_toa,
     &  ray_a0(ip),ray_sb(ip),ray_tb(ip),rc,rc_star,r_sys,ler(ip) 
	enddo

	close(57)
        call system('mv fort.57 wfcld46_tc00_a05_o3')

	stop
	end
	subroutine rad_ray(csp,sza,stcloud,nprs,pscale,rad_a0,sb,tbar)


c       Radiance and T, SB calculations for Rayleigh atmosphere and different pressures: pscale
c       warning: max number of layers < ( MAXLAYER = 120 )
	character csp*4


C  include file of dimensions and numbers

        INCLUDE '../includes/LIDORT.PARS'

C  input variables


        INCLUDE '../includes/LIDORT_INPUTS.VARS'
        INCLUDE '../includes/LIDORT_L_INPUTS.VARS'

C  include file of bookkeeping variables

        INCLUDE '../includes/LIDORT_BOOKKEEP.VARS'

C  result variables

        INCLUDE '../includes/LIDORT_RESULTS.VARS'
        INCLUDE '../includes/LIDORT_L_RESULTS.VARS'

C  help variables

        INTEGER          STATUS_INPUTCHECK,status_inputread
        INTEGER          STATUS_CALCULATION

c  Additional variables for this test (local and passing parameters)
        character*4 stcloud
	integer  iprs,nprs
        integer          n, ndum, idum, l, nmom,k
	double precision sza,wvlth,psrfc,xno2,tautot
        double precision tray_tot,tno2_tot,tcl_tot,taer_tot
        double precision dum, tot_tau, gas_tau, aer_tau, ray_tau
        double precision cld_tau, depol, ray_2mom, tau_scat, amom(500)
        double precision cld_tau_scat, aer_tau_scat
        double precision gastau_save(100),tottau_save(100)
        double precision pres_save(100),tauscat_save(100)

        double precision pscale(nprs)   ! Cloud pressure levels for LER calculation
       

c	surface albedo values for T and Sb calculations
        double precision r1,r2,d1,d2,rad_a0(nprs),sb(nprs),tbar(nprs)
        double precision surf_albedo(4),isave(4),ler(4)
 	data surf_albedo/0.0,0.05,0.15,0.8/	! surf_albedo(1) should be 0.0 for correct T and Sb calculation !
        if (surf_albedo(1) .ne. 0.0 ) then
        write(*,*) ' surf_albedo(1) .ne. 0.0 :',surf_albedo(1)
        stop
	endif

C  Read the LIDORT inputs from file

        CALL LIDORT_L_INPUT_MASTER
     &   ( 'ray_rad.inp','LIDORT_rad_ray_LOGFILE',status_inputread)
        if ( status_inputread.ne.lidort_success)
     &              stop'badread rad_ray.inp'

C Fix the solar zenith angle
        
c        SUN0 = sza              ! solar zenith angle in degrees
        BEAM_SZAS(1) = sza

C  Direct nadir output - settings
c        DO_NO_AZIMUTH         = .TRUE.
c        N_USER_STREAMS = 1
c        USER_ANGLES_INPUT(1) = 0.0
c        N_USER_RELAZMS = 1
c       USER_RELAZMS(1) = 0.0              ! Dummy

C  Rayleigh cycle
C  ==============

	do iprs=1,nprs		!  cycle through surf. pressure (scaling tau_ray in each layer)


C  Set up optical property inputs, and their linearizations (derivatives)

C  read moments

        open(1,file='C1_pmom_lesha',status='old')
        read(1,*)nmom
        do n = 0, nmom - 1
          read(1,*) ndum, amom(n+1)
        enddo
        close(1)

C  read atmosphere

        HEIGHT_GRID(0)   = 85.0D0       ! heights in [km] descending from TOA

        open(1,file='Layers_'//csp//stcloud,status='old')
c       open(1,file='Layers_no2.'//stcloud,status='old')
        read(1,*)
	read(1,*) wvlth,psrfc,depol,xno2,tautot,dum,nlayers
        WRITE(57,*) 'wvlth psrfc  rho  xNO2[atmcm] tautot deltau nolyr'
	WRITE(57,*) wvlth,pscale(iprs)*psrfc,depol,xno2,tautot,dum,
     &                                                     nlayers
        ray_2mom = ( one - depol ) / ( two + depol )
        read(1,*)
        read(1,*)tray_tot,taer_tot,tno2_tot,tcl_tot,dum,dum,idum
        WRITE(57,*) 'tray   taer   tno2   tcld  '
        WRITE(57,*)pscale(iprs)*tray_tot,taer_tot,tno2_tot,tcl_tot
        read(1,*)
        read(1,*)dum,dum
        read(1,*)

        do n = 1, nlayers
          read(1,*)idum,HEIGHT_GRID(n),dum,
     &           ray_tau, aer_tau, gas_tau, cld_tau, tot_tau
	pres_save(n)=pscale(iprs)*dum
c	  tot_tau=tot_tau - ray_tau*(1.-pscale(iprs) )		! correct tot_tau first using ray_tau unchanged
          ray_tau=pscale(iprs)*ray_tau			! Scale Rayleigh tau according to surface pressure level
          aer_tau=0.0
          cld_tau=0.0
          tot_tau=ray_tau + aer_tau + gas_tau + cld_tau
          deltau_vert_input(n) =  tot_tau			! Layer tau

c						tau scattering
	  cld_tau_scat=0.0
c	  cld_tau_scat=cld_tau*cld_ssa
	  aer_tau_scat=0.0
c	  aer_tau_scat=aer_tau*aer_ssa
          tau_scat = ray_tau + aer_tau_scat + cld_tau_scat

          omega_total_input(n) = tau_scat / tot_tau		! Layer SSA
c         if ( omega_total_input(n) .ge. one ) then
c           omega_total_input(n) = 0.99999d0
c         endif

          gastau_save(n)  = gas_tau
          tottau_save(n)  = tot_tau
          tauscat_save(n) = tau_scat
c						Phase function
          phasmoms_total_input(0,n) =one
          phasmoms_total_input(1,n) =0.0                               
c         phasmoms_total_input(1,n) =cld_tau_scat*amom(2)/tau_scat
          phasmoms_total_input(2,n) = 
     &    (ray_tau*ray_2mom + 0.0                  ) / tau_scat
c    &    (ray_tau*ray_2mom + cld_tau_scat*amom(3) ) / tau_scat
          do l = 3, nmoments_input
            phasmoms_total_input(l,n)=0.0
c           phasmoms_total_input(l,n)=cld_tau_scat*amom(l+1)/tau_scat
          enddo
c						Linearization vrt gas_tau
          layer_vary_flag(n)   = .true.
          layer_vary_number(n) = 1
          l_omega_total_input(1,n)  = - gas_tau / tot_tau
          l_deltau_vert_input(1,n) = + gas_tau / tot_tau
          do l = 0, nmoments_input
            l_phasmoms_total_input(1,l,n) = zero
          enddo
        enddo
        close(1)
        

C  call lidort repeats with different albedos for T and Sb calculation

	write(*,'(a)') 'starting LIDORT_L_MASTER: RAY: Ia, T, SB '
	WRITE(57,* )         'Surface albedo         TOA radiance' 
        do k = 1, 4      

C  prepare surface (Lambertian albedo)
          LAMBERTIAN_ALBEDO=surf_albedo(k) 

          CALL LIDORT_L_MASTER
     &        ( STATUS_INPUTCHECK, STATUS_CALCULATION )
          CALL LIDORT_STATUS
     &        ( STATUS_INPUTCHECK, STATUS_CALCULATION )

          isave(k)=INTENSITY(1,1,UPIDX)
          WRITE(57,*)LAMBERTIAN_ALBEDO,isave(k)                  
	enddo

	rad_a0(iprs)=isave(1)
c	Calculating T and Sb values:
	r1=surf_albedo(3)
        d1=isave(3)-isave(1)	! isave(1)=Ia
	r2=surf_albedo(4)
        d2=isave(4)-isave(1)	! isave(1)=Ia
        sb(iprs)=(r1*d2-r2*d1)/(d2-d1)/r1/r2
	tbar(iprs)=d2*(r2-r1)*d1/(d2-d1)/r1/r2		! CORRECTED
          WRITE(57,*        )
     & 	'psurf=',pscale(iprs)*psrfc,
     &  'Ia=',rad_a0(iprs),' 	Sb=',sb(iprs),'	T=',tbar(iprs)                                                
        do k = 1, 4      
        ler(k)=(isave(k)-isave(1))/
     &         (tbar(iprs)+sb(iprs)*(isave(k)-isave(1)))
          WRITE(57,*        )
     & 	'Surface albedo=',surf_albedo(k),'    LER=',ler(k)                  
	enddo

C  Finish
	enddo	! end of pressure cycle


	return
        END

	subroutine wf(cloud_ssa,csp,sza,salbedo,stcloud,nprs,pscale,isave,tcl_tot)

        integer IB,UM,UA,V
        double precision cloud_ssa,salbedo
	character*4 csp

C  include file of dimensions and numbers

        INCLUDE '../includes/LIDORT.PARS'

C  input variables


        INCLUDE '../includes/LIDORT_INPUTS.VARS'
        INCLUDE '../includes/LIDORT_L_INPUTS.VARS'

C  include file of bookkeeping variables

        INCLUDE '../includes/LIDORT_BOOKKEEP.VARS'

C  result variables

        INCLUDE '../includes/LIDORT_RESULTS.VARS'
        INCLUDE '../includes/LIDORT_L_RESULTS.VARS'

C  help variables

        INTEGER          STATUS_INPUTCHECK,status_inputread
        INTEGER          STATUS_CALCULATION

        character*4 stcloud
	integer  iprs,nprs
        integer          n, ndum, idum, l, nmom,k
	double precision sza,wvlth,psrfc,xno2,tautot
        double precision tray_tot,tno2_tot,tcl_tot,taer_tot
        double precision dum, tot_tau, gas_tau, aer_tau, ray_tau
        double precision cld_tau, depol, ray_2mom, tau_scat, amom(500)
        double precision cld_tau_scat,aer_tau_scat                       
        double precision gastau_save(100),tottau_save(100)
        double precision pres_save(100),tauscat_save(100)
        double precision epsilon

c       Cloud pressure levels for LER calculation

        double precision pscale(nprs)
        double precision isave

C  Read the LIDORT inputs from file

        CALL LIDORT_L_INPUT_MASTER
     &   ( 'wf.inp','LIDORT_wf_LOGFILE',status_inputread)
        if ( status_inputread.ne.lidort_success)stop'badread wf.inp'     


C  Note: NMOMENTS_INPUT should be very large for SS corrections, and must
C        at least 2*NSTREAMS - 1 (delta-M off), 2*NSTREAMS (delta-M on)
                
C Fix the solar zenith angle
        
c        SUN0 = sza              ! solar zenith angle in degrees
        BEAM_SZAS(1) = sza

C  Direct nadir output - settings
c        DO_NO_AZIMUTH         = .TRUE.
c        N_USER_STREAMS = 1
c        USER_ANGLES_INPUT(1) = 0.0
c        N_USER_RELAZMS = 1
c       USER_RELAZMS(1) = 0.0              ! Dummy

C  Cycle loop
C  ==========

	do iprs=nprs,nprs		!  cycle through surf. pressure (scaling tau_ray in each layer)
c	do iprs=1,nprs		!  cycle through surf. pressure (scaling tau_ray in each layer)


C  Set up optical properties and their derivatives

C  read cloud moments

        open(1,file='C1_pmom_lesha',status='old')
        read(1,*)nmom
        do n = 0, nmom - 1
          read(1,*) ndum, amom(n+1)
        enddo
        close(1)

C  read cloud atmosphere

        HEIGHT_GRID(0)   = 85.0D0       ! heights in [km] descending from TOA

        open(1,file='Layers_'//csp//stcloud,status='old')
        read(1,*)
	read(1,*) wvlth,psrfc,depol,xno2,tautot,dum,nlayers
        WRITE(57,*) 'wvlth psrfc  rho  xNO2[atmcm] tautot deltau nolyr'
	WRITE(57,*) wvlth,pscale(iprs)*psrfc,depol,xno2,tautot,dum,
     &                                                     nlayers
        ray_2mom = ( one - depol ) / ( two + depol )
        read(1,*)
        read(1,*)tray_tot,taer_tot,tno2_tot,tcl_tot,dum,dum,idum
        WRITE(57,*) 'tray   taer   tno2   tcl_tot  '
        WRITE(57,*)pscale(iprs)*tray_tot,taer_tot,tno2_tot,tcl_tot
        read(1,*)
        read(1,*)dum,dum
        read(1,*)

        do n = 1, nlayers
          read(1,*)idum,HEIGHT_GRID(n),dum,
     &           ray_tau, aer_tau, gas_tau, cld_tau, tot_tau
        pres_save(n)=pscale(iprs)*dum
c         tot_tau=tot_tau - ray_tau*(1.-pscale(iprs) )      	! correct tot_tau first using unchanged ray_tau
          ray_tau=pscale(iprs)*ray_tau         			! Scale Rayleigh tau according to surface pressure level
          tot_tau = ray_tau + aer_tau + gas_tau + cld_tau
          deltau_vert_input(n) =  tot_tau			! Layer tau

c						tau_scat
          cld_tau_scat=cld_tau*cloud_ssa
          aer_tau_scat=aer_tau*cloud_ssa
          tau_scat = ray_tau + aer_tau_scat + cld_tau_scat

          omega_total_input(n) = tau_scat / tot_tau		!  Layer SSA
c         if ( omega_total_input(n) .gt. 0.99999999d0 ) then
c           omega_total_input(n) = 0.99999999d0
c         endif

          gastau_save(n)  = gas_tau
          tottau_save(n)  = tot_tau
          tauscat_save(n) = tau_scat

c						Phase function
          phasmoms_total_input(0,n) = one
          phasmoms_total_input(1,n) = cld_tau_scat * amom(2)/tau_scat
          phasmoms_total_input(2,n) = 
     &    (ray_tau*ray_2mom + cld_tau_scat*amom(3) ) / tau_scat
          do l = 3, nmoments_input
            phasmoms_total_input(l,n)=cld_tau_scat*amom(l+1)/tau_scat
          enddo
c						Layer WF vrt gas_tau
          layer_vary_flag(n)   = .true.
          layer_vary_number(n) = 1
          l_omega_total_input(1,n) = - gas_tau / tot_tau
          l_deltau_vert_input(1,n) = + gas_tau / tot_tau
          do l = 0, nmoments_input
           l_phasmoms_total_input(1,l,n) = zero
          enddo
        enddo
        close(1)
        
C  prepare surface (Lambertian albedo)

        Lambertian_ALBEDO = salbedo

C  call LIDORT with WF

        write(*,'(a)')   ' starting LIDORT_L_MASTER '
        CALL LIDORT_L_MASTER
     &        ( STATUS_INPUTCHECK, STATUS_CALCULATION )
        CALL LIDORT_STATUS
     &        ( STATUS_INPUTCHECK, STATUS_CALCULATION )

        isave = INTENSITY(1,1,DNIDX)



C  output

      IF ( STATUS_CALCULATION .NE. LIDORT_SUCCESS .OR.
     &     STATUS_INPUTCHECK  .EQ. LIDORT_SERIOUS ) THEN
       WRITE(*,*)' No output written to file'
      ELSE
       WRITE(*,*)' output written to file TEST3_RADIANCES.out'
       OPEN(1,FILE='TEST3_RADIANCES.out',STATUS='unknown')
       write(1,'(/a/a//a/)')'Upwelling TOA radiances:',
     &           '------------------------',
     &           'solar angle  view angle   Azimuth      Intensity'
       DO IB = 1, NBEAMS
        DO UM = 1, N_USER_STREAMS
         DO UA = 1, N_USER_RELAZMS
          V = UMOFF(IB,UM) + UA
          WRITE(1,'(1x,f7.2,5x,f7.2,5x,f8.2,2x,1pe16.8)')
     & BEAM_SZAS(IB),USER_ANGLES_INPUT(UM),USER_RELAZMS(UA),
     &  INTENSITY(1,V,DNIDX)
         ENDDO
        ENDDO
       ENDDO
       write(1,'(/a/a//a/)')'Upwelling BOA radiances:',
     &           '------------------------',
     &           'solar angle  view angle   Azimuth      Intensity'
       DO IB = 1, NBEAMS
        DO UM = 1, N_USER_STREAMS
         DO UA = 1, N_USER_RELAZMS
          V = UMOFF(IB,UM) + UA
          WRITE(1,'(1x,f7.2,5x,f7.2,5x,f8.2,2x,1pe16.8)')
     & BEAM_SZAS(IB),USER_ANGLES_INPUT(UM),USER_RELAZMS(UA),
     &  INTENSITY(1,V,DNIDX)
         ENDDO
        ENDDO
       ENDDO
       CLOSE(1)
      ENDIF

C  finish

          WRITE(57,*        )
     & 	'SZA=',sza                      
          WRITE(57,*        )
     & 	'Surface presure=',pscale(iprs)*psrfc
          WRITE(57,*        )
     & 	'FLUX_FACTOR=',FLUX_FACTOR       
          WRITE(57,*        )
     & 	'Surface albedo=',LAMBERTIAN_ALBEDO,'    TOA radiance=',isave


C  call FD LIDORT repeats

        DO_SIMULATION_ONLY      = .TRUE.
        DO_LINEARIZATION        = .FALSE.
        DO_ATMOS_LINEARIZATION  = .FALSE.

        epsilon = 0.01d0	! this works only for tcl=0 and 1
c       epsilon = 0.001d0	! this works only for tcl=0 and 1

        do k = 1, nlayers
         write(*,*)'doing pert ',k
         do n = 1, nlayers
          if ( k.eq.n ) then
            gas_tau = gastau_save(n) * (1.0d0 + epsilon)
            tot_tau = tottau_save(n) + gastau_save(n) * epsilon
          else
            gas_tau = gastau_save(n)
            tot_tau = tottau_save(n)
          endif
c          write(*,*)k,n,tauscat_save(n),tottau_save(n)

          deltau_vert_input(n) =  tot_tau
          omega_total_input(n) = tauscat_save(n) / tot_tau
c         if ( omega_total_input(n) .gt. 0.99999999d0 ) then
c           omega_total_input(n) = 0.99999999d0
c         endif
          layer_vary_flag(n)   = .false.
          layer_vary_number(n) = 0
          l_omega_total_input(1,n) = zero
          l_deltau_vert_input(1,n) = zero
          do l = 0, nmoments_input
            l_phasmoms_total_input(1,l,n) = zero
          enddo
         enddo

          CALL LIDORT_L_MASTER
     &        ( STATUS_INPUTCHECK, STATUS_CALCULATION )
          CALL LIDORT_STATUS
     &        ( STATUS_INPUTCHECK, STATUS_CALCULATION )

          WRITE(57,'(1p12e14.5)')
     &   half*(HEIGHT_GRID(k)+HEIGHT_GRID(k-1)),pres_save(k),
     &   gastau_save(k),INTENSITY(1,1,DNIDX),
     &  (INTENSITY(1,1,DNIDX)-isave)/EPSILON,
     &   ATMOSWF(1,K,1,1,DNIDX),
     &  (INTENSITY(1,1,DNIDX)-isave)/EPSILON/gastau_save(k)/
     &                                                  isave,
     &   ATMOSWF(1,K,1,1,DNIDX)/isave/gastau_save(k)

        enddo


C  Finish

	enddo	! end of pressure cycle

	return
        END

